==> clj-util-0.1.0-20151223.130951-1.jar: util/jsonapi.clj <==
(ns util.jsonapi
  "用于处理jsonapi规范。请参考http://jsonapi.org/。"
  (:require [ring.util.response :as r]
            [cheshire.core :as json]
            [ring.util.request :refer [request-url]]
            [clojure.walk :refer [keywordize-keys]]
            [schema.core :as s]))

;;
;; 定义schema
;;

(def AnyMap {s/Any s/Any})

(def StrOrKey (s/cond-pre s/Str s/Keyword))

(def StrOrMap (s/cond-pre s/Str AnyMap))

(def Attributes
  {StrOrKey s/Any})

(def Links
  {StrOrKey StrOrMap})

(def Relationship
  {(s/optional-key :links) Links
   (s/optional-key :data) s/Any
   (s/optional-key :meta) s/Any})

(def Relationships
  {StrOrKey Relationship})

(def ResourceIdentifier
  {:id StrOrKey
   :type StrOrKey})

(def Resource
  {:id StrOrKey
   :type StrOrKey
   (s/optional-key :attributes) Attributes
   (s/optional-key :links) Links
   (s/optional-key :relationships) Relationships})

(def PrimaryData
  (s/cond-pre [Resource] (s/maybe Resource)))

;;
;; 定义jsonapi操作
;;

(def content-type "application/vnd.api+json; charset=utf-8")

(defn document
  "创建包含主数据的jsonapi文档"
  ([data] {:data data})
  ([data links] {:data data, :links links})
  ([data links included]
   {:data data
    :links links
    :included included}))

(defn error-document
  "创建包含错误信息的jsonapi文档。"
  [error]
  ;; TODO
  )

(s/defn links :- Links
        "创建顶层links对象。request之后，接受偶数个参数，形如: name1 link1 name2 link2 ..."
        [request & name-link-seq]
        (let [links {:self (request-url request)}]
          (if name-link-seq
            (keywordize-keys (apply assoc links name-link-seq))
            links)))

(defn resource-identifier
  "创建资源标识。"
  [id type]
  {:id (str id), :type (name type)})

(defn resource
  "创建资源。"
  ([id type] (resource-identifier id type))
  ([id type attributes] (assoc
                          (resource-identifier id type)
                          :attributes attributes))
  ([id type attributes links] (assoc
                                (resource id type attributes)
                                :links links)))

(defn relationship?
  "判断是否relationship对象。"
  [relationship]
  (and (associative? relationship)
       (some #{:links :data :meta} (keys relationship))))

(defn relationships
  "创建relationships。接受偶数个参数，形如: name1 relationship1 name2 relationship2 ..."
  [name relationship & nrs]
  (let [relationships (if nrs
                        (apply assoc {name relationship} nrs)
                        {name relationship})]
    {:relationships (keywordize-keys relationships)}))

(defn json-response
  "将body转换为json字符串"
  [response]
  (update-in response [:body] (fn [body]
                                (cond
                                  (nil? body) body
                                  (string? body) body
                                  :else (json/encode body)))))

(defn response
  "处理响应map：
  1、增加content-type
  2、如果(associative? body)为真，将body转换为json字符串"
  [response]
  (->
    response
    (r/content-type content-type)
    json-response))

==> clj-util-0.1.0-20160105.093257-2.jar: util/jsonapi.clj <==
(ns util.jsonapi
  "用于处理jsonapi规范。请参考http://jsonapi.org/。"
  (:require [ring.util.response :as r]
            [cheshire.core :as json]
            [ring.util.request :refer [request-url]]
            [clojure.walk :refer [keywordize-keys]]
            [schema.core :as s]))

;;
;; 定义schema
;;

(def AnyMap {s/Any s/Any})

(def StrOrKey (s/cond-pre s/Str s/Keyword))

(def StrOrMap (s/cond-pre s/Str AnyMap))

(def Attributes
  {StrOrKey s/Any})

(def Links
  {StrOrKey StrOrMap})

(def Relationship
  {(s/optional-key :links) Links
   (s/optional-key :data) s/Any
   (s/optional-key :meta) s/Any})

(def Relationships
  {StrOrKey Relationship})

(def ResourceIdentifier
  {:id StrOrKey
   :type StrOrKey})

(def Resource
  {:id StrOrKey
   :type StrOrKey
   (s/optional-key :attributes) Attributes
   (s/optional-key :links) Links
   (s/optional-key :relationships) Relationships})

(def PrimaryData
  (s/cond-pre [Resource] (s/maybe Resource)))

;;
;; 定义jsonapi操作
;;

(def content-type "application/vnd.api+json; charset=utf-8")

(defn document
  "创建包含主数据的jsonapi文档"
  ([data] {:data data})
  ([data links] {:data data, :links links})
  ([data links included]
   {:data data
    :links links
    :included included}))

(defn error-document
  "创建包含错误信息的jsonapi文档。"
  [error]
  ;; TODO
  )

(s/defn links :- Links
        "创建顶层links对象。request之后，接受偶数个参数，形如: name1 link1 name2 link2 ..."
        [request & name-link-seq]
        (let [links {:self (request-url request)}]
          (if name-link-seq
            (keywordize-keys (apply assoc links name-link-seq))
            links)))

(defn resource-identifier
  "创建资源标识。"
  [id type]
  {:id (str id), :type (name type)})

(defn resource
  "创建资源。"
  ([id type] (resource-identifier id type))
  ([id type attributes] (assoc
                          (resource-identifier id type)
                          :attributes attributes))
  ([id type attributes relationships] (assoc
                                        (resource id type attributes)
                                        :relationships relationships))
  ([id type attributes relationships links] (assoc
                                              (resource id type attributes relationships)
                                              :links links)))

(defn relationship?
  "判断是否relationship对象。"
  [relationship]
  (and (associative? relationship)
       (some #{:links :data :meta} (keys relationship))))

(defn relationships
  "创建relationships。接受偶数个参数，形如: name1 relationship1 name2 relationship2 ..."
  [name relationship & nrs]
  (let [relationships (if nrs
                        (apply assoc {name relationship} nrs)
                        {name relationship})]
    {:relationships (keywordize-keys relationships)}))

(defn json-response
  "将body转换为json字符串"
  [response]
  (update-in response [:body] (fn [body]
                                (cond
                                  (nil? body) body
                                  (string? body) body
                                  :else (json/encode body)))))

(defn response
  "处理响应map：
  1、增加content-type
  2、如果(associative? body)为真，将body转换为json字符串"
  [response]
  (->
    response
    (r/content-type content-type)
    json-response))
